@@ -1,6 +1,6 @@
 ;;; nyacc/lang/tcl/parser.scm - parse tcl code
 
-;; Copyright (C) 2018,2020 Matthew R. Wette
+;; Copyright (C) 2018,2020,2022 Matthew R. Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -31,30 +31,34 @@
 
 (define-module (nyacc lang tcl parser)
   #:export (read-command
-	    read-tcl-stmt
-	    read-tcl-file
-	    ;; debugging:
-	    split-body
-	    cnvt-tree
-	    cnvt-args
-	    splice-xtail
-	    tclme
-	    )
+            read-tcl-stmt
+            read-tcl-file
+            ;; debugging:
+            split-body
+            fix-expr-string
+            splice-xtail
+            cnvt-args
+            num-string
+            vec-string
+            cnvt-cond-tail
+            tcl-res-cmds
+            nxify-command
+            cnvt-tree
+            *trace-tcl-parsing*)
   #:use-module ((nyacc lex) #:select (read-basic-num cnumstr->scm c:ir))
   #:use-module (nyacc lang sx-util)
+  #:use-module (nyacc lang util)
   #:use-module (sxml match)
   #:use-module ((srfi srfi-1) #:select (fold-right))
   #:use-module (ice-9 match))
 
 (use-modules (ice-9 pretty-print))
 (define pp pretty-print)
-(define (sf fmt . args) (apply simple-format #t fmt args))
-(define (zf fmt . args) #f)
-(define db zf)
 
-(define (echo obj)
-  (sf " ~S\n" obj)
-  obj)
+(define *trace-tcl-parsing* (make-parameter #f))
+(define (db fmt . args)
+  (when (*trace-tcl-parsing*)
+    (apply simple-format #t fmt args)))
 
 (define char-hex?
   (let ((cs (string->char-set "0123456789abcdefABCDEF")))
@@ -113,6 +117,7 @@
 ;; BUG in read-frag? if ending found two levels deep need to pop two levels ??
 ;; in read-frag: maybe tagged as string when should not be
 
+;; return '() on EOF
 (define (read-command port)
   ;; This is a bit of a hack job.
   (letrec
@@ -121,7 +126,7 @@
        (error
 	(lambda (fmt . args)
 	  (with-input-from-port port
-	    (lambda () (apply report-error port fmt args)))))
+	    (lambda () (apply report-error fmt args)))))
        
        (foldin
 	(lambda (word word-list)
@@ -172,6 +177,17 @@
 	      (read-char port)
 	      (loop wordl (peek-char port)))
 
+             ((char=? #\\ ch)           ; maybe escaped newline
+              (read-char port)
+              (let ((c2 (peek-char port)))
+                (cond
+                 ((char=? #\newline c2)
+                  (read-char port)
+                  (loop wordl (peek-char port)))
+                 (else
+                  (unread-char (read-escape) port)
+                  (loop wordl (peek-char port))))))
+
 	     ((char=? #\" ch)
 	      (read-char port)
 	      (db "C: \" .. reading word\n")
@@ -326,9 +342,11 @@
        ((char-set-contains? cs:ct nxt) (read-char port)))
       (unless (eof-object? cmd)
         (set-source-properties! cmd (source-properties (cadr cmd))))
-      cmd)
-    ;; or (swallow (read-cmmd cs:ct))
-    ))
+      (if (eof-object? cmd) '() cmd))))
+  
+(define (read-tcl-string str)
+  (call-with-input-string str
+    (lambda (port) (read-tcl-stmt port (current-module)))))
 
 ;; @deffn {Procedure} split-body body
 ;; For the string @var{body} which is known to be interpreted as a sequence
@@ -341,9 +359,8 @@
    (call-with-input-string body
      (lambda (port)
        (let loop ((cmd (read-command port)))
-         ;;(pp cmd)
 	 (cond
-	  ((eof-object? cmd) '())
+	  ((null? cmd) '())
 	  ((null? (cdr cmd)) (loop (read-command port)))
 	  ((eq? cmd noop-command) (loop (read-command port)))
 	  (else (cons cmd (loop (read-command port))))))))))
@@ -496,12 +513,20 @@
      . ,(lambda (tree) `(continue)))
     ("expr"
      . ,(lambda (tree) `(expr . ,(splice-xtail (sx-tail tree 2)))))
+    ("for"
+     . ,(lambda (tree)
+	  (sxml-match tree
+	    ((command (string "for") (string ,init) (string ,cond)
+                      (string ,next) (string ,body))
+             ;;(pp (read-tcl-string (string-append "expr " cond)))
+	     `(for  ,(read-tcl-string init)
+                    ,(read-tcl-string (string-append "expr " cond))
+                    ,(read-tcl-string next) ,(split-body body)))
+	    (,_ (report-error "usage: for init cond next body")))))
     ("format"
      . ,(lambda (tree) `(format . ,(sx-tail tree 2))))
     ("if"
      . ,(lambda (tree)
-	  ;;(sf "tree:\n") (pp tree) ;;(quit)
-	  ;;(sf "============\n")
 	  (sxml-match tree
 	    ((command (string "if") (string ,cnd) (string "then")
 		      (string ,bdy) . ,rest)
@@ -510,9 +535,7 @@
 	    ((command (string "if") (string ,cnd) (string ,bdy) . ,rest)
 	     `(if ,(fix-expr-string cnd) ,(split-body bdy)
 		  . ,(cnvt-cond-tail rest)))
-	    (,_ (report-error "usage: if cond then else")))
-	  ;;) (quit)
-	  ))
+	    (,_ (report-error "usage: if cond then else")))))
     ("incr"
      . ,(lambda (tree) `(incr ,@(sx-tail tree 2))))
     ("proc"
@@ -522,10 +545,7 @@
 	    ((command (string "proc") (string ,name) (string ,args)
 		      (string ,body))
 	     `(proc ,name ,(cnvt-args args) ,(split-body body)))
-            ;;((command (string "proc") (word x) (string ,args) (string ,body))
-	    ;;(,_ (report-error "usage: proc name args body"))
-            (,_ #f)
-            )))
+            (,_ #f))))
     ("return"
      . ,(lambda (tree)
 	  `(return ,(or (sx-ref tree 2) ""))))
@@ -540,8 +560,10 @@
 	     (if (pair? rest)
 		 `(set-indexed (string ,name) (word ,indx) . ,rest)
 		 `(deref-indexed ,name (word ,indx))))
-	    ;;(,_ (report-error "can't handle this yet")))))
 	    (,_ `(set . ,(sx-tail tree 2))))))
+    ("source"
+     . ,(lambda (tree)
+          `(source ,(sx-ref tree 2))))
     ("while"
      . ,(lambda (tree)
 	  (sxml-match tree
@@ -572,7 +594,6 @@
 		     (lambda (proc) (proc tree)))))
     (if (and (pair? repl) (eq? 'command (car repl))) (error "coding error"))
     repl))
-(export nxify-command)
 
  ;; convert special commands and words into nx-tcl syntax
 ;; @example
@@ -582,26 +603,29 @@
   (letrec
       ((cnvt-elt
 	(lambda (tree)
-	  (if (string? tree) tree
-	      (sxml-match tree
-		((command . ,rest)
-		 (or (and=> (nxify-command tree) cnvt-tree)
-		     (let* ((tail0 (sx-tail tree))
-			    (tail1 (cnvt-tail tail0)))
-		       (if (eq? tail1 tail0) tree `(command . ,tail1)))))
-		((string ,val)
-		 (or (num-string val) tree))
-		((word (string ,val))
-		 (cnvt-tree (sx-ref tree 1)))
-		((word . ,rest)
-		 (let* ((tail0 (sx-tail tree))
-			(tail1 (cnvt-tail tail0)))
-		   (if (eq? tail1 tail0) tree `(word . ,tail1))))
-		(,_
-		 (let* ((tag (sx-tag tree))
-			(tail0 (sx-tail tree))
-			(tail1 (cnvt-tail tail0)))
-		   (if (eq? tail1 tail0) tree (cons tag tail1))))))))
+          (cond
+           ((null? tree) tree)
+           ((string? tree) tree)
+           (else
+	    (sxml-match tree
+	      ((command . ,rest)
+	       (or (and=> (nxify-command tree) cnvt-tree)
+		   (let* ((tail0 (sx-tail tree))
+			  (tail1 (cnvt-tail tail0)))
+		     (if (eq? tail1 tail0) tree `(command . ,tail1)))))
+	      ((string ,val)
+	       (or (num-string val) tree))
+	      ((word (string ,val))
+	       (cnvt-tree (sx-ref tree 1)))
+	      ((word . ,rest)
+	       (let* ((tail0 (sx-tail tree))
+		      (tail1 (cnvt-tail tail0)))
+		 (if (eq? tail1 tail0) tree `(word . ,tail1))))
+	      (,_
+	       (let* ((tag (sx-tag tree))
+		      (tail0 (sx-tail tree))
+		      (tail1 (cnvt-tail tail0)))
+		 (if (eq? tail1 tail0) tree (cons tag tail1)))))))))
        (cnvt-tail
 	(lambda (tail)
 	  (if (null? tail) tail
@@ -616,6 +640,9 @@
 			(cons head1 tail1))))))))
     (cnvt-elt tree)))
 
+(define (read-n-cnvt-cmmd port)
+  (cnvt-tree (read-command port)))
+
 ;; @deffn {Procedure} read-tcl-stmt port env
 ;; Guile extension language routine to read a single statement.
 ;; @end deffn
@@ -632,13 +659,20 @@
 (define (read-tcl-file port env)
   "- Procedure: read-tcl-file port env
      Read a Tcl file.  Return a SXML tree."
-  (if (eof-object? (peek-char port))
-      (read-char port)
-      (cons
-       'unit
-       (let loop ((cmd (read-command port)))
-	 (if (eof-object? cmd) '()
-	     (let ((cmd1 cmd) (cmd2 (cnvt-tree cmd)))
-	       (cons (cnvt-tree cmd) (loop (read-command port)))))))))
+  (cons 'unit
+        (let loop ((cmd (read-n-cnvt-cmmd port)))
+          (if (null? cmd) '()
+              (cons
+               (sx-match cmd
+                 ((source (string ,name))
+                  (dynamic-wind
+                    (lambda () (push-input (open-input-file name)))
+                    (lambda () (read-tcl-file (current-input-port) env))
+                    (lambda () (pop-input))))
+                 ((source (command . ,args))
+                  (sferr "nx-tcl parser: can't handle complex source")
+                  (pperr args) (quit))
+                 (,_ cmd))
+               (loop (read-n-cnvt-cmmd port)))))))
 
 ;; --- last line ---
